from unittest import TestCase

import operator

from rpnpy import Calculator


class TestCalculator(TestCase):

    def testEmptyStack(self):
        """A calculator must start life with an empty stack"""
        self.assertEqual([], Calculator().stack)

    def testPushNumber(self):
        """Pushing a number must work as expected"""
        c = Calculator()
        c.execute('4')
        (result,) = c.stack
        self.assertEqual(4, result)

    def testPushString(self):
        """Pushing a string must work as expected"""
        c = Calculator()
        c.execute("'4'")
        (result,) = c.stack
        self.assertEqual('4', result)

    def testPushAbs(self):
        """Pushing the operator.abs function must work as expected"""
        c = Calculator()
        c.execute('operator.abs /!')
        (result,) = c.stack
        self.assertIs(operator.abs, result)

    def testAdd(self):
        """Adding must work"""
        c = Calculator()
        c.execute('3')
        c.execute('4')
        c.execute('+')
        (result,) = c.stack
        self.assertEqual(7, result)

    def testRegister(self):
        """Registering and calling a new function must work"""
        def double(n):
            return 2 * n

        c = Calculator()
        c.register('module', 'double', double, 1)
        c.execute('3')
        c.execute('double')
        (result,) = c.stack
        self.assertEqual(6, result)



#!/usr/bin/env python

from __future__ import print_function

import sys
import os
import readline
import argparse

from rpnpy import Calculator


def setupReadline():
    # Readline code from https://docs.python.org/3.7/library/readline.html
    histfile = os.path.join(os.path.expanduser('~'), '.pycalc_history')

    try:
        readline.read_history_file(histfile)
        historyLen = readline.get_current_history_length()
    except FileNotFoundError:
        open(histfile, 'wb').close()
        historyLen = 0

    try:
        readline.append_history_file
    except AttributeError:
        # We won't be able to save readline history. This can happen on
        # Python 3.5 at least - not sure why.
        pass
    else:
        import atexit

        def saveHistory(prevHistoryLen, histfile):
            newHistoryLen = readline.get_current_history_length()
            readline.set_history_length(1000)
            readline.append_history_file(newHistoryLen - prevHistoryLen,
                                         histfile)

        atexit.register(saveHistory, historyLen, histfile)


def parseArgs():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        description=(
            'An RPN calculator for Python. Reads commands from standard input '
            'or interactively with a read-eval-print loop and (optionally) '
            'writes the final stack to standard output.'))

    parser.add_argument(
        '--prompt', default='--> ',
        help='The prompt to print at the start of each line when interactive.')

    parser.add_argument(
        '--separator',
        help=('The character to use to split lines of standard input into '
              'separate commands (unless --noSplit is given).'))

    parser.add_argument(
        '--debug', action='store_true', default=False,
        help='Print verbose information about how commandsa are run.')

    parser.add_argument(
        '--version', action='store_true', default=False,
        help='Print the version number and exit.')

    parser.add_argument(
        '--noSplit', action='store_false', default=True, dest='split',
        help=('If given, do not split lines read from standard input into '
              'separate commands, treat each line as an entire command.'))

    parser.add_argument(
        '--noPrint', action='store_false', default=True, dest='print',
        help=('If given, do not print the stack after processing all commands '
              'from standard input.'))

    return parser.parse_args()


def stdin(calc, split, separator, print_):
    """
    Read and execute commands from stdin.

    @param calc: A C{Calculator} instance.
    @param split: If C{True}, split lines.
    @param separator: The C{str} to split lines on (if C{split} is C{True}
        or C{None} to split on whitespace.
    @param print_: If C{True}, print the stack after all commands are run.
    """
    for line in sys.stdin:
        commands = line.split(separator) if split else (line,)
        for command in commands:
            try:
                calc.execute(command)
            except EOFError:
                break
    if print_ and calc.stack:
        calc.printStack(-1 if len(calc) == 1 else None)


def repl(calc, prompt):
    """
    Interactive read-eval-print loop.

    @param calc: A C{Calculator} instance.
    @param prompt: The C{str} prompt to print at the start of each line.
    """
    while True:
        try:
            calc.execute(input(prompt))
        except KeyboardInterrupt:
            print()
        except EOFError:
            break


if __name__ == '__main__':
    args = parseArgs()

    if args.version:
        from rpnpy import __version__
        print(__version__)
    else:
        setupReadline()
        calc = Calculator(debug=args.debug)

        if os.isatty(0):
            repl(calc, args.prompt)
        else:
            stdin(calc, args.split, args.separator, args.print)



from unittest import TestCase

from rpnpy.io import splitInput


class TestInput(TestCase):
    """Test the splitInput function."""

    def testEmpty(self):
        """Test an empty string."""
        self.assertEqual(('', set(), None), splitInput(''))

    def testComment(self):
        """Test a comment."""
        self.assertEqual(('', set(), None), splitInput('# comment'))

    def testCommentPrecededByWhitespace(self):
        """Test a comment preceeded by whitespace."""
        self.assertEqual(('', set(), None), splitInput('  # comment'))

    def testLoneSlash(self):
        """Test a slash by itself."""
        self.assertEqual(('/', set(), None), splitInput('/'))

    def testLoneSlashPrecededByWhitespace(self):
        """Test a slash by itself with some leading whitespace."""
        self.assertEqual(('/', set(), None), splitInput(' /'))

    def testLoneSlashFollowedByWhitespace(self):
        """Test a slash by itself with some trailing whitespace."""
        self.assertEqual(('/', set(), None), splitInput('/ '))

    def testLoneSlashSurroundedByWhitespace(self):
        """Test a slash by itself with surrounding whitespace."""
        self.assertEqual(('/', set(), None), splitInput(' / '))

    def testCommandWithNoModifiers(self):
        """Test a command with no modifiers"""
        self.assertEqual(('hey', set(), None), splitInput('hey'))

    def testCommandWithNoModifiersPrecededByWhitespace(self):
        """Test a command with no modifiers preceded by whitespace"""
        self.assertEqual(('hey', set(), None), splitInput(' hey'))

    def testCommandWithNoModifiersFollowedByWhitespace(self):
        """Test a command with no modifiers followed by whitespace"""
        self.assertEqual(('hey', set(), None), splitInput(' hey'))

    def testCommandWithLeadingSpaceAndNoModifiers(self):
        """Test a command with leading space and no modifiers"""
        self.assertEqual(('hey', set(), None), splitInput(' hey'))

    def testCommandWithTrailingSpaceAndNoModifiers(self):
        """Test a command with trailing space and no modifiers"""
        self.assertEqual(('hey', set(), None), splitInput('hey '))

    def testCommandWithSurroundingSpaceAndNoModifiers(self):
        """Test a command with surrounding space and no modifiers"""
        self.assertEqual(('hey', set(), None), splitInput(' hey '))

    def testCommandWithNoModifiersCaseUnchanged(self):
        """Test that command case is unmodified"""
        self.assertEqual(('HeY', set(), None), splitInput('HeY'))

    def testCommandWithModifiers(self):
        """Test a command with modifiers"""
        self.assertEqual(('hey', set('abc'), None), splitInput('hey /abc'))

    def testCommandWithMixedCaseModifiers(self):
        """Test a command with mixed case modifiers"""
        self.assertEqual(('hey', set('abc'), None), splitInput('hey /AbC'))

    def testCommandWithModifiersNoSpace(self):
        """Test a command with modifiers but no space before the /"""
        self.assertEqual(('hey', set('abc'), None), splitInput('hey/abc'))

    def testCommandWithCount(self):
        """Test a command with a count"""
        self.assertEqual(('hey', set(), 101), splitInput('hey /101'))

    def testCommandWithCountThenModifiers(self):
        """Test a command with a count then modifiers"""
        self.assertEqual(('hey', set('abc'), 101), splitInput('hey /101abc'))

    def testCommandWithCountThenSpaceAndModifiers(self):
        """Test a command with a count then a space then modifiers"""
        self.assertEqual(('hey', set('abc'), 101), splitInput('hey /101 abc'))

    def testCommandWithModifiersThenCount(self):
        """Test a command with modifiers then a count"""
        self.assertEqual(('hey', set('abc'), 101), splitInput('hey /abc101'))

    def testCommandWithModifiersThenSpaceAndCount(self):
        """Test a command with modifiers, a space, and then a count"""
        self.assertEqual(('hey', set('abc'), 101), splitInput('hey /abc 101'))

    def testCommandWithModifiersSurroundingCount(self):
        """Test a command with modifiers before and after a count"""
        self.assertEqual(('hey', set('abcd'), 16), splitInput('hey /ab 16 cd'))
